import os

# Disable SAM spyware, should be opt-in
os.environ['SAM_CLI_TELEMETRY'] = '0'


def ssm_lookup(scope, parameter_name):
    from aws_cdk import aws_ssm
    return aws_ssm.StringParameter.value_from_lookup(scope, parameter_name=parameter_name)



from aws_cdk import (
    aws_secretsmanager,
    core
)


class SecretGenerator(core.Construct):
    def __init__(self, scope: core.Construct, id: str, secret_name: str, template: str = None, key: str = None) -> None:
        """Provides a generate pseudo-random password

        Args:
            scope (core.Construct): [description]
            id (str): [description]
            secret_name (str): [description]
            template (str, optional): [description]. Defaults to None.
            key (str, optional): [description]. Defaults to None.
        """
        super().__init__(scope, id)
        self.secret = aws_secretsmanager.Secret(
            self,
            id,
            generate_secret_string=aws_secretsmanager.SecretStringGenerator(
                secret_string_template=template,
                generate_string_key=key,
                password_length=24,
                exclude_characters='"@/\$'
            ),
            secret_name=secret_name
        )

    def to_string(self):
        return core.SecretValue(self.secret.secret_value.to_string())



import typing
from aws_cdk import (
    aws_codebuild as cb
)


def buildenv(environment_variables: dict) -> typing.Dict[str, cb.BuildEnvironmentVariable]:
    """Facilitate Codebuild environmemt creation
    Simply pass your dict to be turned a env variabltes
    For values that starts with 'aws:sm:', the prefix will be striped off
     and the Codebuild env variable type wil be set to SECRETS_MANAGER.

    Args:
        environment_variables (dict): key/value store to turn into Codebuild Env

    Returns:
        typing.Dict[cb.BuildEnvironmentVariable]: [description]
    """
    envs = dict()
    for env, value in environment_variables.items():
        if isinstance(value, str) and value.startswith('aws:sm:'):
            envs[env] = cb.BuildEnvironmentVariable(
                value=value.replace('aws:sm:', ''),
                type=cb.BuildEnvironmentVariableType.SECRETS_MANAGER
            )
        else:
            envs[env] = cb.BuildEnvironmentVariable(value=value)
    return envs


def load_buildspec(specfile: str='buildspec.yml') -> cb.BuildSpec:
    """Load a buildspec yaml file

    Args:
        specfile ([type]): [description]

    Returns:
        [type]: [description]
    """
    import yaml
    with open(specfile, encoding="utf8") as fp:
        bsfile = fp.read()
        bs = yaml.safe_load(bsfile)
        return cb.BuildSpec.from_object(value=bs)



from aws_cdk import (
    aws_lambda,
    aws_stepfunctions as sfn,
    aws_stepfunctions_tasks as sfn_tasks,
    core
)


class Stepfunctions(core.Construct):
    statemachine: sfn.IStateMachine = None
    start: sfn.IChainable = None

    def __init__(self, scope: core.Construct, id: str, *, start: sfn.IChainable=None) -> None:
        super().__init__(scope, id)

        if not start:
            start = sfn.Pass(self, 'Pass')
        self.start = start

    def machine(self, name: str='stateMachine', timeout: int=1) -> sfn.StateMachine:
        self.statemachine = sfn.StateMachine(
            self, name,
            definition=self.start,
            timeout=core.Duration.minutes(timeout)
        )
        return self.statemachine

    def _wait(self, path:str="$.wait_time") -> sfn.Wait:
        return sfn.Wait(
            self, 'waiter',
            time=sfn.WaitTime.seconds_path(path=path)
        )

    def _invoke_lambda(self, name: str, fx: aws_lambda.IFunction=None,
        code: aws_lambda.Code=None, handler: str=None, runtime=aws_lambda.Runtime.PYTHON_3_7,
        **invoke_attr) -> sfn_tasks.LambdaInvoke:
        if not fx:
            fx = aws_lambda.Function(
                self, "fxi_{}".format(name),
                code=code,
                handler=handler,
                runtime=runtime
            )
        return sfn_tasks.LambdaInvoke(
            self, "{}".format(name),
            lambda_function=fx,
            **invoke_attr
        )



from aws_cdk import (
    aws_codestarconnections as csc,
    core
)


class GithubConnection(core.Construct):
    connection = csc.CfnConnection

    def __init__(self, scope: core.Construct, id: str, connection_name: str, host_arn: str=None, *, **connection_args) -> None:
        super().__init__(scope, id)
        if host_arn:
            connection_args['host_arn'] = host_arn
        self.connection = csc.CfnConnection(
            self, 'github-connection',
            connection_name=connection_name,
            provider_type='GitHub',
            **connection_args
        )



from aws_cdk import (
    aws_secretsmanager,
    aws_iam,
    core
)
from aviv_cdk import secretsmanager


class SecurityCredentials(core.Construct):
    user: aws_iam.IUser

    def __init__(self, scope: core.Construct, id: str, user: aws_iam.IUser=None) -> None:
        super().__init__(scope, id)
        self.user = user

    def console_password(self, template: str = None, key: str = None) -> core.SecretValue:
        secgen = secretsmanager.SecretGenerator(
            self, 'console-password',
            secret_name=f"iam/user/{self.user.user_name}/password",
            template=template,
            key=key
        )
        return secgen.to_string()
 
    def access_key(self):
        self.accesskey = aws_iam.CfnAccessKey(self, 'key', user_name=self.user.user_name)
        aws_secretsmanager.CfnSecret(
            self, 'access-key',
            name=f"iam/user/{self.user.user_name}/accesskey",
            secret_string=f"{{\"AccessKeyId\": \"{self.accesskey.ref}\", "\
                f"\"SecretAccessKey\": \"{self.accesskey.attr_secret_access_key}\"}}"
        )
        return self.accesskey.attr_secret_access_key



import typing
from aws_cdk import (
    aws_codepipeline_actions,
    pipelines,
    core
)

from . import (
    codepipeline,
    sources
)

class PipelineStack(core.Stack):
    pipeline: codepipeline.Pipeline=None
    _pipelines: pipelines.CdkPipeline=None
    stages: typing.Dict[str, codepipeline.cp.IStage]={}

    def __init__(
            self, scope: core.Construct, id: str, *,
            pipeline_attr: dict={},
            connections: dict=None,
            auto_sourcebuild: bool=False,
            **kwargs) -> None:
        super().__init__(scope=scope, id=id, **kwargs)
        self.pipeline = codepipeline.Pipeline(self, 'pipeline', **pipeline_attr)
        if connections:
            self.pipeline.connections = connections
        if auto_sourcebuild:
            self.source()
            self.build()

    @property
    def cdk_pipelines(self) -> pipelines.CdkPipeline:
        if not self._pipelines:
            self._pipelines = pipelines.CdkPipeline(
                self, 'pipelines',
                cloud_assembly_artifact=self.pipeline.assembly,
                code_pipeline=self.pipeline
            )
        return self._pipelines

    def source(self, action_name: str=None, repository: typing.Union[str, sources.SourceRepositoryAttrs]=None, connections: dict=None) -> aws_codepipeline_actions.Action:
        """Add an action to watch a git reporitory/branch to the Source stage

        Args:
            action_name (str, optional): Name your source action (and artifact). Defaults to 'source'.
            repository (typing.Union[str, sources.SourceRepositoryAttrs], optional): The Git(hub) repository
              you are sourcing. Can be either an url or a dict with repo/owner/branch. Defaults to current repo/origin@branch.
            connections (dict, optional): AWS/Github org connections by name/codestar.connection_arn.

        Returns:
            aws_codepipeline_actions.Action: [description]
        """
        if connections:
            self.pipeline.connections = connections
        if 'Source' not in self.stages:
            self.stages['Source'] = self.pipeline.add_stage(stage_name='Source')
        action = self.pipeline.source(action_name=action_name, repository=repository)
        self.stages['Source'].add_action(action=action)
        return action

    def build(self, action_name: str=None, **build_args) -> aws_codepipeline_actions.Action:
        """Add an action to the Build stage

        Args:
            action_name (str, optional): Name your build action/artifact. Defaults to 'build'.

        Returns:
            aws_codepipeline_actions.Action: a Codepipeline / CodeBuildAction
        """
        if 'Build' not in self.stages:
            self.stages['Build'] = self.pipeline.add_stage(stage_name='Build')
        if 'input' not in build_args and 'sources' not in build_args:
            source = self.stages['Source'].actions[0].action_properties.action_name
            build_args['input'] = self.pipeline.artifacts[source]
            if not action_name:
                action_name = f"{source}-build"
        action = self.pipeline.build(action_name, **build_args)
        self.stages['Build'].add_action(action=action)
        return action



import sys
import logging
from aws_cdk import (
    aws_events,
    aws_events_targets,
    aws_lambda,
    aws_ssm,
    core
)


class Construct(core.Construct):
    _layers = dict()

    def __init__(self,  scope: core.Construct, id: str, *, lambdas: dict=None, layers: dict=None):
        super().__init__(scope, id)
        # Process and save AWS Lambda layers
        for layer_name, layer_attr in layers.items():
            self._layers[layer_name] = self.layer(layer_name, **layer_attr)
        # Process and save AWS Lambda Functions
        for fx_name, fx_attr in lambdas.items():
            self.lambda(fx_name, **fx_attr)

    def lambda(self, id: str, *, **attr):
        if 'layers' in attr:
            for layer in attr['layers']:
                if layer in self._layers:
                    layer = self._layers[layer]
        fx = aws_lambda.Function(self, id, **attr)
        core.CfnOutput(self, f"{id}-cfn", value=fx.function_arn)
        return fx

    def layer(self, id: str, *, remote_account_grant: bool=False, ssm_param: bool=True, **attr):
        layv = aws_lambda.LayerVersion(self, id **attr)
        if remote_account_grant:
            layv.add_permission('remote-account-grant', account_id='*')
        if ssm_param:
            aws_ssm.StringParameter(
                self, f"{id}-ssm",
                string_value=layv.layer_version_arn,
                parameter_name=f"/layers/{id.replace('-', '/')}"
            )
        core.CfnOutput(self, f"{id}-cfn", value=layv.from_layer_version_arn)
        return layv
    
    def cron(self, id: str, cron: aws_events.Schedule, *, targets: list=[], event: dict=None, target_args: dict=None, event_attr: dict=None):
        if event:
            event = aws_events.RuleTargetInput.from_object(event)
        if targets:
            for target in targets:
                target = aws_events_targets.LambdaFunction(target, event=event, **target_args)
        return aws_events.Rule(self, id, schedule=cron, targets=targets, **event_attr)


def inline_code(filepath: str) -> str:
    """Streamline file into a string for a simple AWS Lambda

    Args:
        filepath ([type]): [description]

    Returns:
        [type]: [description]
    """
    with open(filepath, encoding="utf8") as fp:
        code = fp.read()
    if sys.getsizeof(code) > 4096:
        logging.warning(f"Code inline size if > 4096: {sys.getsizeof(code)}")
    return code



import logging
import typing

SourceRepositoryAttrs = typing.Dict[typing.Literal['owner', 'repo', 'branch'], str]
GitRepositoryInfo = typing.Dict[typing.Literal['url', 'branch'], str]

def git_repository_info() -> GitRepositoryInfo:
    import subprocess
    cmd = lambda input: subprocess.check_output(input, shell=True).decode('UTF-8').rstrip()
    url=cmd("git remote get-url origin")
    if url.endswith('.git'):
        url = url.replace('.git', '')

    return dict(
        url=url,
        branch=cmd("git branch --show-current")
    )


def github_url_split(url: str, branch: str='main') -> SourceRepositoryAttrs:
    """Splits a https github url to return a dict with:
    - owner     Github organization
    - repo      the git repository
    - branch    the branch

    Args:
        url (str): a https://github.com/your-org/myrepo
        branch (str, optional): [description]. Defaults to 'main'.

    Returns:
        dict: owner/repo/branch
    """
    if not url.startswith('https://github.com/'):
        logging.warning(f"Not an https Github URL: {url}")
    return git_url_split(url, branch)


def git_url_split(url: str, branch: str='main') -> SourceRepositoryAttrs:
    if url.startswith('git@github.com:'):
        url = url.replace('git@github.com:', '')

    repo_attrs = dict(branch=branch)
    # If we have a specific branch
    if url.find('@') > 0:
        branchsplit = url.split('@')
        url = branchsplit[0]
        if branchsplit[1]:
            repo_attrs['branch'] = branchsplit[1]

    if url.endswith('.git'):
        url = url.replace('.git', '')
    urlsplit = url.split('/')
    repo_attrs['owner'] = urlsplit[-2]
    repo_attrs['repo'] = urlsplit[-1]
    return repo_attrs



#!/usr/bin/env python3
import click
import json
from aws_parsecf.functions import Functions
# whatever...
Functions.REF_PSEUDO_FUNCTIONS['AWS::Partition'] = lambda self: 'aws'
def localImportValue(self, value):
    stackfx = value.split('lambdasExportsOutputFnGetAtt')
    fxarn = stackfx[1].split('Arn')
    return fxarn[0]

Functions.stuffit = localImportValue
Functions.MAP['Fn::ImportValue'] = 'stuffit'
# OR stuff directly with all Fn::ImportValue
#Function._import_value_cache = dict()

import aws_parsecf


@click.argument('template', type=click.types.STRING, required=True, default='template.json')
@click.command(short_help='Extract an AWS StepFunctions StateMachine from a CDK/CFN template')
def cli(template: str):
    with open(template, 'r') as f:
        # Extract and generate CFN template parameters
        tpl_raw = json.load(f)
        params = dict()

        # Cleanup CDK specific stuff
        if 'Rules' in tpl_raw and 'CheckBootstrapVersion' in tpl_raw['Rules']:
            del tpl_raw['Rules']['CheckBootstrapVersion']
        if 'Parameters' in tpl_raw and 'BootstrapVersion' in tpl_raw['Parameters']:
            del tpl_raw['Parameters']['BootstrapVersion']

        # for k, v in tpl_raw['Parameters'].items():
        #     if 'Description' in v and v['Description'].find('"') >= 0:
        #         params[k] = v['Description'].split('"')[1]
        #     if str(k).find("S3Bucket") >= 0:
        #         params[k] = "cdk.out/asset||" + params[k]
        # Parse CFN template
        tpl = aws_parsecf.loads_json(json.dumps(tpl_raw), default_region='eu-west-1', parameters=params)

    # Scan resources and print out StateMachine(s)
    for k, r in tpl['Resources'].items():
        if r['Type'] != 'AWS::StepFunctions::StateMachine':
            continue
        # Fixup non existing local attributes
        definition = r['Properties']['DefinitionString'].replace("UNKNOWN ATT: ", "").replace(".Arn", "")
        # print("{}\n{}".format(k, definition))
        jsondef = json.loads(definition)
        print(json.dumps(jsondef, indent=4))


if __name__ == "__main__":
    cli()



import typing
from aws_cdk import (
    aws_codebuild as cb,
    aws_codepipeline as cp,
    aws_codepipeline_actions as cpa,
    aws_codestarconnections as csc,
    aws_s3,
    aws_secretsmanager as sm,
    aws_ssm,
    core
)
from . import (
    codebuild,
    sources
)


class Pipeline(cp.Pipeline):
    assembly: cp.Artifact
    __artifacts: dict={}
    __connections: dict={}

    def __init__(self, scope, id: str, *,
            connections: dict=None,
            **pipeline_args) -> None:

        if connections:
            self.connections = connections
        self.assembly = cp.Artifact('main')
        super().__init__(scope, id, **pipeline_args)

    @property
    def artifacts(self) -> dict:
        return self.__artifacts

    @artifacts.setter
    def artifacts(self, value: dict={}) -> None:
        self.__artifacts = value

    @property
    def connections(self) -> dict:
        return self.__connections

    @connections.setter
    def connections(self, connections: dict) -> None:
        for cname, connection_arn in connections.items():
            if connection_arn.startswith('aws:ssm:'):
                connection_arn = aws_ssm.StringParameter.value_from_lookup(
                    self, parameter_name=connection_arn.replace('aws:ssm:', '')
                )
            self.__connections[cname] = connection_arn

    def source(self, action_name: str=None, repository: typing.Union[str, sources.SourceRepositoryAttrs]=None, **source_args) -> cpa.BitBucketSourceAction:
        # Pick url / branch from the 'current' git clone
        if not repository:
            info = sources.git_repository_info()
            repository = f"{info.get('url')}@{info.get('branch')}"
        if isinstance(repository, str):
            repository = sources.git_url_split(repository)

        if not action_name:
            action_name = f"{repository['repo']}@{repository['branch']}"
        self.artifacts[action_name] = cp.Artifact()
        # repository['owner'] MUST match an existing self.connections
        connection_arn = self.connections[repository['owner']]
        return cpa.BitBucketSourceAction(
            connection_arn=connection_arn,
            output=self.artifacts[action_name],
            code_build_clone_output=True,
            **repository,
            action_name=action_name,
            **source_args
        )

    def build_project(self, id: str, *, specfile: str='buildspec.yml', **project_args) -> cb.Project:
        if not 'build_spec' in project_args:
            project_args['build_spec'] = codebuild.load_buildspec(specfile)
        return cb.Project(
            self, id,
            environment=cb.BuildEnvironment(
                build_image=cb.LinuxBuildImage.AMAZON_LINUX_2_3
            ),
            **project_args
        )

    def build(self, action_name: str, sources: list=[], project_args: dict={}, **build_args) -> cpa.CodeBuildAction:
        self.artifacts[action_name] = [self.assembly]
        project = self.build_project(f"{action_name}-project", **project_args)

        if 'environment_variables' in build_args:
            build_args['environment_variables'] = self.process_environment_variables(
                environment_variables=build_args['environment_variables'], project=project
            )
        if sources:
            build_args['input'] = sources.pop(0)
            build_args['extra_inputs'] = sources

        action = cpa.CodeBuildAction(
            action_name=action_name,
            project=project,
            outputs=self.artifacts[action_name],
            **build_args
        )
        return action

    def process_environment_variables(self, environment_variables: dict, project: cb.Project) -> typing.Dict[str, cb.BuildEnvironmentVariable]:
        environment_variables = codebuild.buildenv(environment_variables)
        for name, var in environment_variables.items():
            if var.type == cb.BuildEnvironmentVariableType.SECRETS_MANAGER:
                sec = sm.Secret.from_secret_name_v2(self, f"{name}-secenv", secret_name=var.value)
                sec.grant_read(project)
        return environment_variables



import typing
import os
from aws_cdk import (
    aws_lambda,
    aws_stepfunctions,
    aws_stepfunctions_tasks,
    aws_events,
    aws_events_targets,
    core
)

LAMBDAS_RUNTIME = os.environ.get('LAMBDAS_RUNTIME', aws_lambda.Runtime.PYTHON_3_7)
LAMBDAS_PATH = os.environ.get('LAMBDAS_PATH', './lambdas')


class StateMachine(core.Construct):
    _sm: aws_stepfunctions.IStateMachine = None
    _smprops: dict = {}
    lambdas_assets: aws_lambda.Code
    lambdas_runtime: aws_lambda.Runtime

    def __init__(self, scope: core.Construct, id: str, *, lambdas_path=LAMBDAS_PATH, lambdas_runtime=LAMBDAS_RUNTIME, **attr) -> None:
        super().__init__(scope, id, **attr)
        self.lambdas_runtime = lambdas_runtime
        self.assets = aws_lambda.Code.asset(lambdas_path)

    @property
    def sm(self) -> aws_stepfunctions.StateMachine:
        if not self._sm:
            self.sm = aws_stepfunctions.StateMachine(self, 'sm', **self.smprops)
        return self._sm

    @sm.setter
    def sm(self, value: aws_stepfunctions.StateMachine):
        self._sm = value

    @property
    def smprops(self) -> dict:
        if not self._smprops:
            self._smprops['definition'] = aws_stepfunctions.Pass(self, 'pass')
        return self._smprops

    @smprops.setter
    def smprops(self, value: typing.Union[aws_stepfunctions.StateMachineProps, dict]):
        if isinstance(value, aws_stepfunctions.StateMachineProps):
            self._smprops = value.__dict__['_values']
        self._smprops = value

    def launch(self, statemachine_props: aws_stepfunctions.StateMachineProps=None) -> aws_stepfunctions.StateMachine:
        self.smprops = statemachine_props
        core.CfnOutput(self, 'sm-arn', value=self.sm.state_machine_arn)
        return self.sm

    def lambda_task(
            self,
            name: str,
            fx: aws_lambda.IFunction=None,
            fx_attr: aws_lambda.FunctionProps=None,
            invoke: dict={}):
        if not fx:
            fx_attr = aws_lambda.FunctionProps(
                code=self.assets,
                handler=f"{name}.handler",
                runtime=self.lambdas_runtime
            )
            fx = aws_lambda.Function(
                self, "fxi_{}".format(name), **fx_attr.__dict__['_values']
            )

        return aws_stepfunctions_tasks.LambdaInvoke(
            self, "{}".format(name),
            lambda_function=fx,
            **invoke
        )

    def cron(self, name: str, cron: dict, *, input: aws_events.RuleTargetInput=None):
        rule = aws_events.Rule(self, name, schedule=aws_events.Schedule.cron(**cron))
        target = aws_events_targets.SfnStateMachine(
            self.sm,
            input=input
        )
        rule.add_target(target)



import setuptools
import aviv_cdk

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="aviv-cdk", # Replace with your own username
    version=aviv_cdk.__version__,
    author="Jules Clement",
    author_email="jules.clement@aviv-group.com",
    description="Aviv CDK Python library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/aviv-group/aviv-cdk-python",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    py_modules=[
        'bin.aws_local',
        'bin.sfn_extract'
    ],
    data_files=[
        ("share/aviv-cdk/cfn-resources", ["lambdas/cfn_resources/requirements.txt"]),
        ("share/aviv-cdk/iam-idp", [
            "lambdas/iam_idp/saml.py",
            "samples/idp/buildspec.yml",
            "lambdas/cfn_resources/requirements.txt"
        ])
    ],
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'aviv-aws=bin.aws_local:cli',
            'aviv-cdk-sfn-extract=bin.sfn_extract:cli'
        ],
    },
    install_requires=[
         "boto3>=1.14",
         "click>=7.1",
         "aws-parsecf>=1.1"
         "aws-cdk-core>=1.94",
         "aws-cdk-aws-iam>=1.94",
         "aws-cdk-aws-s3>=1.94",
         "aws-cdk-aws-lambda>=1.94",
         "aws-cdk-aws-events>=1.94",
         "aws-cdk-aws-events-targets>=1.94",
         "aws-cdk-aws-ssm>=1.94",
         "aws-cdk-aws-secretsmanager>=1.94"
   ],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],

    extras_require={
        "cicd": [
            "aws-cdk-aws-cloudformation",
            "aws-cdk-aws-codebuild",
            "aws-cdk-aws-codecommit",
            "aws-cdk-aws-codepipeline",
            "aws-cdk-aws-codepipeline-actions",
            "aws-cdk-aws-codestarconnections",
            "aws-cdk-pipelines",
            "pyyaml>=5.4"
        ],
        "nextstep": [
            "aws-cdk-aws-stepfunctions",
            "aws-cdk-aws-stepfunctions-tasks"
        ],
        "data": [
            "aws-cdk-glue",
            "aws-cdk-athena"
        ]
    },
    python_requires='>=3.8',
    use_2to3=False,
    zip_safe=False
)



#!/usr/bin/env python3
from os import kill
import time
import json
import os
import shlex
import subprocess
import logging
import click

# disable SAM spyware
os.environ['SAM_CLI_TELEMETRY'] = '0'

PROCESSES = []
def popen(cmd, **args):
    if not 'shell' in args or args['shell'] is not True:
        cmd = shlex.split(cmd)
    p = subprocess.Popen(cmd, **args)
    PROCESSES.append(p)
    return p


def getLambdas(tpl: str):
    with open(tpl) as f:
        tpl = json.load(f)
    resources = []

    for n, r in tpl['Resources'].items():
        if r['Type'] != 'AWS::Lambda::Function':
            continue
        resources.append(n)
    return resources


def killAll():
    for p in PROCESSES:
        p.kill()
    click.secho("\nAll done byebye!\n", bold=True)


@click.option('--profile', '-p', type=click.types.STRING, default='local')
@click.group(help="Aviv AWS Client")
def cli():
    click.secho("Aviv AWS Client", bold=True)


@cli.command(short_help='Install AWS stuff locally')
def install():
    click.secho("AWS tools install and setup helper")
    click.secho("Press 'y' + [enter] to install or just [enter] to pass\n", dim=True)
    if input("- Install AWS codebuild local? [y]") == 'y':
        codebuild_url = 'https://raw.githubusercontent.com/aws/aws-codebuild-docker-images/master/local_builds/codebuild_build.sh'
        os.system('cd /usr/local/bin && wget {} && chmod +x codebuild_build.sh'.format(codebuild_url))


@cli.command(short_help='AWS local codebuild')
def build():
    cbdockervs = 'aws/codebuild/standard:4.0'
    args = ' -i {} -a build/ -s .'.format(cbdockervs)
    # if os.path.exists('.env'):
    #     args += ' -e .env'
    os.system('echo codebuild_build.sh {}'.format(args))


@click.argument('template', type=click.types.STRING, default='template.json')
@click.option('--synth', '-s', help='Do a fresh cdk synth', is_flag=True, default=False)
@click.option('--sfn', '-S', help='AWS StepFunctionsLocal', is_flag=True, default=False)
# @click.option('--sfn-jar', help='AWS StepFunctionsLocal.jar file path', type=click.types.STRING, default=AWS_STEPFUNCTIONS_JAR)
@click.option('--api', '-a', help='SAM local start-api', is_flag=True, default=False)
@click.option('--debug', '-d', is_flag=True, default=False)
@cli.command(short_help='AWS local backends')
def daemons(template, profile, synth, sfn, api, debug):
    click.secho(" >  SFN local StateMachine", bold=True)
    args = f" --profile {profile}"
    if debug:
        args += " --debug"
    #     click.secho("Template: {}".format(template), dim=True)
    #     if profile:
    #         click.secho("Profile: {}".format(profile), dim=True)
    #     print()

    # if synth:
    #     click.secho("CDK synth refreshing templates", dim=True)
    #     os.system('cdk synth {} --no-staging'.format(args))
    #     print()

    popen("sam local start-lambda {} -t {}".format(args, template))
    click.secho("SAM local start-lambda", dim=True)
    for i in range(10):
        if PROCESSES[-1].poll():
            logging.error("start-lambda FAILED pid:{}".format(PROCESSES[-1].pid))
            killAll()
            exit(42)
        time.sleep(.5)

    if api:
        click.secho("\nStarting AWS SAM Local API...", dim=True)
        popen("sam local start-api {} -t {}".format(args, template))
        time.sleep(5)
        if PROCESSES[-1].poll():
            logging.error("sam local start-api - pid: {}\n".format(PROCESSES[-1].pid))

    if sfn:
        click.secho("\nStarting AWS SFN Local...", dim=True)
        click.secho('Hit [CTRL]-C to exit and kill all processes\n', bold=True)

        envstr = ''
        for e, v in dict(
            SAM_CLI_TELEMETRY=0,
            AWS_ACCOUNT_ID="123456789012",
            AWS_DEFAULT_REGION="us-east-1",
            LAMBDA_ENDPOINT="http://host.docker.internal:3001/").items():
            envstr += ' --env {}={}'.format(e, v)
        os.system("docker run -ti {} -p 8083:8083 amazon/aws-stepfunctions-local".format(envstr))


    for p in PROCESSES:
        p.kill()
    click.secho("\nAll done byebye!\n", bold=True)


def _help_sm():
    for i, l in enumerate(['- Create a StateMachine', '- Start it', '+ [EXECID] - Describe execution', '+ [EXECID] - Get exec. history']):
        click.secho(" [{}] {}".format(i + 1, l))
    click.secho(" [0] || [exit] - Delete StateMachine and exit", dim=True)

@click.argument('template', type=click.types.STRING, required=True, default='template.json')
@click.option('--profile', '-p', type=click.types.STRING, default='local')
@click.option('--debug', '-d', is_flag=True, default=False)
@cli.command(short_help='Run stuff locally')
def sm(template, profile, debug):
    click.secho(" >  run SFN StateMachine")

    sfnargs = ' --profile {} --endpoint http://127.0.0.1:8083'.format(profile)
    if debug:
        sfnargs += " --debug"

    sfnargscreate = sfnargs + " --name localstatem --role-arn 'arn:aws:iam::123456789012:role/DummyRole'"

    # Print state machine
    smid = "arn:aws:states:us-east-1:123456789012:stateMachine:localstatem"
    execid="arn:aws:states:us-east-1:123456789012:execution:localstatem:0ade304c-9aa1-4c2f-90f0-4d86059XXXXX"

    line = ''
    while line != 'exit':
        if line == '0':
            break
        elif line == '1':
            os.system("aviv-cdk-sfn-extract {} | xargs -0 aws stepfunctions create-state-machine {} --definition ".format(
                template, sfnargscreate
            ))
        elif line == '2':
            os.system("aws stepfunctions start-execution {} --state-machine {}".format(sfnargs, smid))
            # TODO: capture SM execId (--output text | cut -f1)
            # execid='arn:local:exec'
        elif isinstance(line, str) and line.startswith('3 '):
            execid = line[2:]
            os.system("aws stepfunctions describe-execution {} --execution-arn {}".format(sfnargs, execid))
        elif isinstance(line, str) and line.startswith('4 '):
            execid = line[2:]
            os.system("aws stepfunctions get-execution-history {} --execution-arn {}".format(sfnargs, execid))
        else:
            _help_sm()
        line = input("AVIV AWS:sm $ ")

    os.system("aws stepfunctions delete-state-machine {} --state-machine-arn {}".format(sfnargs, smid))
    click.secho("\nAll gone byebye!", dim=True)


def _help_run(lambdas: list):
    print("Run Lambda:")
    for i, l in enumerate(lambdas):
        print(" [{}] - {}".format(i + 1, l))
    click.secho(" [0] || [exit] - Exit", dim=True)

@click.argument('template', type=click.types.STRING, required=True, default='template.json')
@click.option('--profile', '-p', type=click.types.STRING, default='local')
@click.option('--debug', '-d', is_flag=True, default=False)
@cli.command(short_help='Run stuff locally')
def run(template, profile, debug):
    click.secho("=== Local invoke ===")
    lbds = getLambdas(template)
    
    line = ''
    while line != 'exit':
        if line == '0':
            break
        elif line.isdigit() and lbds[int(line) - 1]:
            os.system("echo {{\"status\": \"start\"}} | sam local invoke --profile {} --template {} {}".format(
                profile, template, lbds[int(line) - 1]
            ))
        else:
            _help_run(lbds)
        line = input("AVIV AWS:run $ ")

    click.secho("\nAll gone byebye!", dim=True)

if __name__ == "__main__":
    cli()



import typing
from aws_cdk import (
    aws_ssm,
    core
)


class Stack(core.Stack):
    def __init__(self,
            scope: typing.Optional[core.Construct]=None,
            id: typing.Optional[str]=None,
            *,
            StackProps: core.StackProps=None) -> None:

        super().__init__(
            scope,
            id,
            **StackProps
        )



import sys
sys.path.append('/opt')  # nopep8
import logging

def handler(request, context):
  logging.warning(request)

  import requests
  import boto3
  from botocore.exceptions import ClientError
  from cfn_resource_provider import ResourceProvider

  class SAMLProvider(ResourceProvider):
    """
    Generic Cloudformation custom resource provider for Auth0 resources.
    from https://github.com/binxio/cfn-saml-provider
    """
    def __init__(self):
      super(SAMLProvider, self).__init__()
      self.request_schema = {
        "type": "object",
        "oneOf": [
          {
          "properties": {
            "Name": {"type": "string","description": "of the saml provider"},
            "Metadata": {"type": "string", "description": "of the saml provider"}
          },
          "required": ["Name", "Metadata"]
          },
          {
          "properties": {
            "Name": {"type": "string", "description": "of the saml provider"},
            "URL": { "type": "string", "description": "pointing to the SAML Metadata Document"}
          },
          "required": ["Name", "URL"]
          }
        ]
      }
      self.iam = boto3.client('iam')

    @property
    def custom_cfn_resource_name(self):
      return 'Custom::SAMLProvider'

    def get_metadata(self):
      logging.warning("RP: {}".format(self.properties))
      metadata = self.get('Metadata', None)
      if metadata != None:
        return None, metadata
      try:
        response = requests.get(self.get('URL'))
        metadata = response.text
        if response.status_code == 200:
          return None, metadata
      except requests.exceptions.RequestException as e:
        return '{}'.format(e), None

      return 'url %s returned status code %d, %s'.format(self.get('URL'), response.status_code, response.text), None

    def create(self):
      err, metadata = self.get_metadata()
      if err:
        self.physical_resource_id = 'could-not-create'
        self.fail(err)
        return
      try:
        response = self.iam.create_saml_provider(Name=self.get('Name'), SAMLMetadataDocument=metadata)
        self.physical_resource_id = response['SAMLProviderArn']
      except ClientError as e:
        self.physical_resource_id = 'could-not-create'
        self.fail('{}'.format(e))

    def update(self):
      err, metadata = self.get_metadata()
      if err:
        self.fail(err)
        return
      try:
        response = self.iam.update_saml_provider(SAMLProviderArn=self.physical_resource_id, SAMLMetadataDocument=metadata)
        self.physical_resource_id = response['SAMLProviderArn']
      except ClientError as e:
        self.fail('{}'.format(e))

    def delete(self):
      import re
      if re.match(r'arn:aws[-a-z]*:iam::[0-9]*:saml-provider/.*$', self.physical_resource_id):
        try:
          response = self.iam.delete_saml_provider(SAMLProviderArn=self.physical_resource_id)
          self.physical_resource_id = response['SAMLProviderArn']
        except ClientError as e:
          self.fail('{}'.format(e))

  provider = SAMLProvider()
  return provider.handle(request, context)



__version__='0.1.2'


def json_load(filename: str) -> dict:
    import json
    with open(filename) as cfgfile:
        return json.load(cfgfile)

def load_yaml(filename: str) -> object:
    import yaml
    with open(filename, encoding="utf8") as fp:
        with fp.read() as bsfile:
            return yaml.safe_load(bsfile)
